@@ -1,5 +1,6 @@
 /* set ellipsoid parameters a and es */
 
+#include <errno.h>
 #include <math.h>
 #include <stddef.h>
 #include <string.h>
@@ -115,9 +116,9 @@ int pj_ellipsoid (PJ *P) {
     if (0 != ellps_spherification (P))
         return 4;
 
-    proj_log_debug (P, "pj_ellipsoid - final: a=%.3f f=1/%7.3f, errno=%d",
+    proj_log_trace (P, "pj_ellipsoid - final: a=%.3f f=1/%7.3f, errno=%d",
                         P->a,  P->f!=0? 1/P->f: 0,  proj_errno (P));
-    proj_log_debug (P, "pj_ellipsoid - final: %s %s %s %s",
+    proj_log_trace (P, "pj_ellipsoid - final: %s %s %s %s",
                         P->def_size?           P->def_size: empty,
                         P->def_shape?          P->def_shape: empty,
                         P->def_spherification? P->def_spherification: empty,
@@ -156,7 +157,14 @@ static int ellps_ellps (PJ *P) {
     err = proj_errno_reset (P);
 
     paralist* new_params = pj_mkparam (ellps->major);
+    if (nullptr == new_params)
+        return proj_errno_set (P, ENOMEM);
     new_params->next = pj_mkparam (ellps->ell);
+    if (nullptr == new_params->next)
+    {
+        pj_dealloc(new_params);
+        return proj_errno_set (P, ENOMEM);
+    }
     paralist* old_params = P->params;
     P->params = new_params;
 
@@ -389,6 +397,9 @@ static int ellps_spherification (PJ *P) {
             return proj_errno_set (P, PJD_ERR_REF_RAD_LARGER_THAN_90);
         t = sin (t);
         t = 1 - P->es * t * t;
+        if (t == 0.) {
+            return proj_errno_set(P, PJD_ERR_INVALID_ECCENTRICITY);
+        }
         if (i==5)   /* arithmetic */
             P->a *= (1. - P->es + t) / (2 * t * sqrt(t));
         else        /* geometric */
