@@ -1,7 +1,7 @@
 /******************************************************************************
  *
  * Project:  PROJ
- * Purpose:  ISO19111:2018 implementation
+ * Purpose:  ISO19111:2019 implementation
  * Author:   Even Rouault <even dot rouault at spatialys dot com>
  *
  ******************************************************************************
@@ -31,6 +31,7 @@
 #endif
 
 #include "proj/common.hpp"
+#include "proj/coordinateoperation.hpp"
 #include "proj/coordinatesystem.hpp"
 #include "proj/crs.hpp"
 #include "proj/datum.hpp"
@@ -57,6 +58,7 @@ using namespace NS_PROJ::crs;
 using namespace NS_PROJ::datum;
 using namespace NS_PROJ::io;
 using namespace NS_PROJ::metadata;
+using namespace NS_PROJ::operation;
 using namespace NS_PROJ::util;
 
 NS_PROJ_START
@@ -271,6 +273,8 @@ DEFINE_WKT_CONSTANT(BASEVERTCRS);
 DEFINE_WKT_CONSTANT(BASEENGCRS);
 DEFINE_WKT_CONSTANT(BASEPARAMCRS);
 DEFINE_WKT_CONSTANT(BASETIMECRS);
+DEFINE_WKT_CONSTANT(VERSION);
+DEFINE_WKT_CONSTANT(GEOIDMODEL);
 
 DEFINE_WKT_CONSTANT(GEODETICCRS);
 DEFINE_WKT_CONSTANT(GEODETICDATUM);
@@ -339,12 +343,12 @@ const UnitOfMeasure UnitOfMeasure::MICRORADIAN("microradian", 1e-6,
 /** \brief Second, unit of measure of type TIME (SI unit). */
 const UnitOfMeasure UnitOfMeasure::SECOND("second", 1.0,
                                           UnitOfMeasure::Type::TIME,
-                                          Identifier::EPSG, "1029");
+                                          Identifier::EPSG, "1040");
 
 /** \brief Year, unit of measure of type TIME */
 const UnitOfMeasure UnitOfMeasure::YEAR("year", 31556925.445,
                                         UnitOfMeasure::Type::TIME,
-                                        Identifier::EPSG, "1040");
+                                        Identifier::EPSG, "1029");
 
 /** \brief Metre per year, unit of measure of type LINEAR. */
 const UnitOfMeasure UnitOfMeasure::METRE_PER_YEAR("metres per year",
@@ -641,4 +645,13 @@ const GeographicCRSNNPtr
 
 // ---------------------------------------------------------------------------
 
+/** \brief Key to set the operation version of a operation::CoordinateOperation
+ *
+ * The value is to be provided as a string.
+ */
+const std::string
+    operation::CoordinateOperation::OPERATION_VERSION_KEY("operationVersion");
+
+// ---------------------------------------------------------------------------
+
 NS_PROJ_END
