@@ -30,7 +30,7 @@
 #include "proj.h"
 #include <errno.h>
 #include "proj_internal.h"
-#include "proj_math.h"
+#include <math.h>
 
 namespace { // anonymous namespace
 enum Mode {
@@ -151,7 +151,7 @@ oblcon:
                 return xy;
             }
             else
-                xy.x = xy.y = 0.;
+                return aeqd_e_forward(lp, P);
         else {
             xy.y = acos(xy.y);
             xy.y /= sin(xy.y);
