@@ -9,7 +9,7 @@ PROJ_HEAD(august, "August Epicycloidal") "\n\tMisc Sph, no inv";
 #define M 1.333333333333333
 
 
-static PJ_XY s_forward (PJ_LP lp, PJ *P) {           /* Spheroidal, forward */
+static PJ_XY august_s_forward (PJ_LP lp, PJ *P) {           /* Spheroidal, forward */
     PJ_XY xy = {0.0,0.0};
 	double t, c1, c, x1, x12, y1, y12;
 	(void) P;
@@ -29,7 +29,7 @@ static PJ_XY s_forward (PJ_LP lp, PJ *P) {           /* Spheroidal, forward */
 
 PJ *PROJECTION(august) {
     P->inv = nullptr;
-    P->fwd = s_forward;
+    P->fwd = august_s_forward;
     P->es = 0.;
     return P;
 }
