@@ -33,11 +33,10 @@
 #include <errno.h>
 #include <math.h>
 
-#include "proj_internal.h"
 #include "proj.h"
 #include "proj_internal.h"
 
-PROJ_HEAD(healpix, "HEAPJ_LPix") "\n\tSph&Ell";
+PROJ_HEAD(healpix, "HEAPJ_LPix") "\n\tSph&Ell\n\trot_xy=";
 PROJ_HEAD(rhealpix, "rHEAPJ_LPix") "\n\tSph&Ell\n\tnorth_square= south_square=";
 
 /* Matrix for counterclockwise rotation by pi/2: */
@@ -57,6 +56,7 @@ namespace { // anonymous namespace
 struct pj_opaque {
     int north_square;
     int south_square;
+    double rot_xy;
     double qp;
     double *apa;
 };
@@ -79,6 +79,12 @@ static double sign (double v) {
     return v > 0 ? 1 : (v < 0 ? -1 : 0);
 }
 
+static PJ_XY rotate(PJ_XY p, double angle) {
+    PJ_XY result;
+    result.x = p.x * cos(angle) - p.y * sin(angle);
+    result.y = p.y * cos(angle) + p.x * sin(angle);
+    return result;
+}
 
 /**
  * Return the index of the matrix in ROT.
@@ -179,45 +185,31 @@ static int in_image(double x, double y, int proj, int north_square,
             {-M_FORTPI,   -M_HALFPI - EPS},
             {-M_HALFPI,   -M_FORTPI - EPS},
             {-3*M_FORTPI, -M_HALFPI - EPS},
-            {-M_PI - EPS, -M_FORTPI}
+            {-M_PI - EPS, -M_FORTPI},
+            {-M_PI - EPS,  M_FORTPI}
         };
         return pnpoly((int)sizeof(healpixVertsJit)/
                       sizeof(healpixVertsJit[0]), healpixVertsJit, x, y);
     } else {
         /**
-         * Assigning each element by index to avoid warnings such as
-         * 'initializer element is not computable at load time'.
-         * Before C99 this was not allowed and to keep as portable as
-         * possible we do it the C89 way here.
          * We need to assign the array this way because the input is
          * dynamic (north_square and south_square vars are unknown at
          * compile time).
          **/
-        double rhealpixVertsJit[12][2];
-        rhealpixVertsJit[0][0]  = -M_PI - EPS;
-        rhealpixVertsJit[0][1]  =  M_FORTPI + EPS;
-        rhealpixVertsJit[1][0]  = -M_PI + north_square*M_HALFPI- EPS;
-        rhealpixVertsJit[1][1]  =  M_FORTPI + EPS;
-        rhealpixVertsJit[2][0]  = -M_PI + north_square*M_HALFPI- EPS;
-        rhealpixVertsJit[2][1]  =  3*M_FORTPI + EPS;
-        rhealpixVertsJit[3][0]  = -M_PI + (north_square + 1.0)*M_HALFPI + EPS;
-        rhealpixVertsJit[3][1]  =  3*M_FORTPI + EPS;
-        rhealpixVertsJit[4][0]  = -M_PI + (north_square + 1.0)*M_HALFPI + EPS;
-        rhealpixVertsJit[4][1]  =  M_FORTPI + EPS;
-        rhealpixVertsJit[5][0]  =  M_PI + EPS;
-        rhealpixVertsJit[5][1]  =  M_FORTPI + EPS;
-        rhealpixVertsJit[6][0]  =  M_PI + EPS;
-        rhealpixVertsJit[6][1]  = -M_FORTPI - EPS;
-        rhealpixVertsJit[7][0]  = -M_PI + (south_square + 1.0)*M_HALFPI + EPS;
-        rhealpixVertsJit[7][1]  = -M_FORTPI - EPS;
-        rhealpixVertsJit[8][0]  = -M_PI + (south_square + 1.0)*M_HALFPI + EPS;
-        rhealpixVertsJit[8][1]  = -3*M_FORTPI - EPS;
-        rhealpixVertsJit[9][0]  = -M_PI + south_square*M_HALFPI - EPS;
-        rhealpixVertsJit[9][1]  = -3*M_FORTPI - EPS;
-        rhealpixVertsJit[10][0] = -M_PI + south_square*M_HALFPI - EPS;
-        rhealpixVertsJit[10][1] = -M_FORTPI - EPS;
-        rhealpixVertsJit[11][0] = -M_PI - EPS;
-        rhealpixVertsJit[11][1] = -M_FORTPI - EPS;
+        double rhealpixVertsJit[][2] = {
+            { -M_PI - EPS,                                  M_FORTPI + EPS },
+            { -M_PI + north_square*M_HALFPI- EPS,           M_FORTPI + EPS },
+            { -M_PI + north_square*M_HALFPI- EPS,           3*M_FORTPI + EPS },
+            { -M_PI + (north_square + 1.0)*M_HALFPI + EPS,  3*M_FORTPI + EPS },
+            { -M_PI + (north_square + 1.0)*M_HALFPI + EPS,  M_FORTPI + EPS},
+            {  M_PI + EPS,                                  M_FORTPI + EPS },
+            {  M_PI + EPS,                                  -M_FORTPI - EPS },
+            { -M_PI + (south_square + 1.0)*M_HALFPI + EPS,  -M_FORTPI - EPS },
+            { -M_PI + (south_square + 1.0)*M_HALFPI + EPS,  -3*M_FORTPI - EPS },
+            { -M_PI + south_square*M_HALFPI - EPS,          -3*M_FORTPI - EPS },
+            { -M_PI + south_square*M_HALFPI - EPS,          -M_FORTPI - EPS },
+            { -M_PI - EPS,                                  -M_FORTPI - EPS }
+        };
 
         return pnpoly((int)sizeof(rhealpixVertsJit)/
                       sizeof(rhealpixVertsJit[0]), rhealpixVertsJit, x, y);
@@ -282,7 +274,7 @@ static PJ_XY healpix_sphere(PJ_LP lp) {
 /**
  * Return the inverse of healpix_sphere().
 **/
-static PJ_LP healpix_sphere_inverse(PJ_XY xy) {
+static PJ_LP healpix_spherhealpix_e_inverse(PJ_XY xy) {
     PJ_LP lp;
     double x = xy.x;
     double y = xy.y;
@@ -452,8 +444,6 @@ static CapMap get_cap(double x, double y, int north_square, int south_square,
 static PJ_XY combine_caps(double x, double y, int north_square, int south_square,
                        int inverse) {
     PJ_XY xy;
-    double v[2];
-    double c[2];
     double vector[2];
     double v_min_c[2];
     double ret_dot[2];
@@ -467,8 +457,8 @@ static PJ_XY combine_caps(double x, double y, int north_square, int south_square
         return xy;
     }
 
-    v[0] = x; v[1] = y;
-    c[0] = capmap.x; c[1] = capmap.y;
+    double v[] = {x, y};
+    double c[] = {capmap.x, capmap.y};
 
     if (inverse == 0) {
         /* Rotate (x, y) about its polar cap tip and then translate it to
@@ -498,11 +488,10 @@ static PJ_XY combine_caps(double x, double y, int north_square, int south_square
     vector_sub(v, c, v_min_c);
     dot_product(tmpRot, v_min_c, ret_dot);
     {
-        double a[2];
-        /* Workaround cppcheck git issue */
-        double* pa = a;
-        pa[0] = -3*M_FORTPI + ((inverse == 0) ? pole : capmap.cn) *M_HALFPI;
-        pa[1] = ((capmap.region == CapMap::north) ? 1 : -1) *M_HALFPI;
+        double a[] = {
+            -3*M_FORTPI + ((inverse == 0) ? pole : capmap.cn) *M_HALFPI,
+            ((capmap.region == CapMap::north) ? 1 : -1) *M_HALFPI
+        };
         vector_add(ret_dot, a, vector);
     }
 
@@ -514,17 +503,22 @@ static PJ_XY combine_caps(double x, double y, int north_square, int south_square
 
 static PJ_XY s_healpix_forward(PJ_LP lp, PJ *P) { /* sphere  */
     (void) P;
-    return healpix_sphere(lp);
+    struct pj_opaque *Q = static_cast<struct pj_opaque*>(P->opaque);
+    return rotate(healpix_sphere(lp), -Q->rot_xy);
 }
 
 
 static PJ_XY e_healpix_forward(PJ_LP lp, PJ *P) { /* ellipsoid  */
     lp.phi = auth_lat(P, lp.phi, 0);
-    return healpix_sphere(lp);
+    struct pj_opaque *Q = static_cast<struct pj_opaque*>(P->opaque);
+    return rotate(healpix_sphere(lp), -Q->rot_xy);
 }
 
 
 static PJ_LP s_healpix_inverse(PJ_XY xy, PJ *P) { /* sphere */
+    struct pj_opaque *Q = static_cast<struct pj_opaque*>(P->opaque);
+    xy = rotate(xy, Q->rot_xy);
+
     /* Check whether (x, y) lies in the HEAPJ_LPix image */
     if (in_image(xy.x, xy.y, 0, 0, 0) == 0) {
         PJ_LP lp;
@@ -533,12 +527,14 @@ static PJ_LP s_healpix_inverse(PJ_XY xy, PJ *P) { /* sphere */
         pj_ctx_set_errno(P->ctx, PJD_ERR_INVALID_X_OR_Y);
         return lp;
     }
-    return healpix_sphere_inverse(xy);
+    return healpix_spherhealpix_e_inverse(xy);
 }
 
 
 static PJ_LP e_healpix_inverse(PJ_XY xy, PJ *P) { /* ellipsoid */
     PJ_LP lp = {0.0,0.0};
+    struct pj_opaque *Q = static_cast<struct pj_opaque*>(P->opaque);
+    xy = rotate(xy, Q->rot_xy);
 
     /* Check whether (x, y) lies in the HEAPJ_LPix image. */
     if (in_image(xy.x, xy.y, 0, 0, 0) == 0) {
@@ -547,7 +543,7 @@ static PJ_LP e_healpix_inverse(PJ_XY xy, PJ *P) { /* ellipsoid */
         pj_ctx_set_errno(P->ctx, PJD_ERR_INVALID_X_OR_Y);
         return lp;
     }
-    lp = healpix_sphere_inverse(xy);
+    lp = healpix_spherhealpix_e_inverse(xy);
     lp.phi = auth_lat(P, lp.phi, 1);
     return lp;
 }
@@ -582,7 +578,7 @@ static PJ_LP s_rhealpix_inverse(PJ_XY xy, PJ *P) { /* sphere */
         return lp;
     }
     xy = combine_caps(xy.x, xy.y, Q->north_square, Q->south_square, 1);
-    return healpix_sphere_inverse(xy);
+    return healpix_spherhealpix_e_inverse(xy);
 }
 
 
@@ -598,7 +594,7 @@ static PJ_LP e_rhealpix_inverse(PJ_XY xy, PJ *P) { /* ellipsoid */
         return lp;
     }
     xy = combine_caps(xy.x, xy.y, Q->north_square, Q->south_square, 1);
-    lp = healpix_sphere_inverse(xy);
+    lp = healpix_spherhealpix_e_inverse(xy);
     lp.phi = auth_lat(P, lp.phi, 1);
     return lp;
 }
@@ -623,6 +619,9 @@ PJ *PROJECTION(healpix) {
     P->opaque = Q;
     P->destructor = destructor;
 
+    double angle = pj_param(P->ctx, P->params,"drot_xy").f;
+    Q->rot_xy = PJ_TORAD(angle);
+
     if (P->es != 0.0) {
         Q->apa = pj_authset(P->es);             /* For auth_lat(). */
         if (nullptr==Q->apa)
