@@ -20,7 +20,7 @@ PROJ_HEAD(rpoly, "Rectangular Polyconic")
 
 #define EPS 1e-9
 
-static PJ_XY s_forward (PJ_LP lp, PJ *P) {           /* Spheroidal, forward */
+static PJ_XY rpoly_s_forward (PJ_LP lp, PJ *P) {           /* Spheroidal, forward */
     PJ_XY xy = {0.0,0.0};
     struct pj_opaque *Q = static_cast<struct pj_opaque*>(P->opaque);
     double fa;
@@ -53,7 +53,7 @@ PJ *PROJECTION(rpoly) {
         Q->fxa = 0.5 / Q->fxb;
     }
     P->es = 0.;
-    P->fwd = s_forward;
+    P->fwd = rpoly_s_forward;
 
     return P;
 }
